@@ -12,7 +12,7 @@ RootDevice::RootDevice(const QString &udi, const QString &parentUdi)
     : Solid::Ifaces::Device()
     , m_udi(udi)
     , m_parentUdi(parentUdi)
-    , m_vendor("KDE")
+    , m_vendor(QStringLiteral("KDE"))
 {
 }
 
@@ -89,3 +89,5 @@ QObject *RootDevice::createDeviceInterface(const Solid::DeviceInterface::Type &)
 {
     return nullptr;
 }
+
+#include "moc_rootdevice.cpp"
